/*
 * Glade - A user interface designer for GTK+ and GNOME
 * Copyright (C) 2014-2016 Juan Pablo Ugarte
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Author: Juan Pablo Ugarte <juanpablougarte@gmail.com>
 * 
 */

#glade-registration {
  background-color: @theme_base_color;
}

#glade-registration .main-box {
  margin: 8px;
}

#glade-registration box.title,
#glade-registration label.title {
  background-color: @theme_selected_bg_color;
  border-radius: 8px;
  padding: 8px;
  box-shadow: 2px 2px 3px gray;
}

#glade-registration label.title {
  color: white;
  font-size: 14pt;
  font-weight: bold;
  text-shadow: 1px 1px 2px black;
}

#glade-registration .info-box,
#glade-registration box.questions > box {
  border: 4px solid;
  border-color: @borders;
  border-radius: 16px;
  padding: 8px;
}

#glade-registration box.textbox {
  border-radius: 16px;
  color: @theme_bg_color;
  background-color: @theme_fg_color;
  box-shadow: 2px 2px 4px gray;
  padding: 8px;
  margin: 0px 16px;
}

#glade-registration box.textbox .textbox-title {
  font-size: 12pt;
  font-weight: bold;
}

#glade-registration box.questions > box > label {
  font-size: 10pt;
  font-weight: bold;
  padding: 0px 0px 8px 0px;
  text-shadow: 1px 1px @theme_bg_color;
}

#glade-registration box.statusbar {
  border-top-right-radius: 1ex;
  background: @theme_bg_color;
}

